@@ -9,10 +9,11 @@ pub mod uvfits;
 use std::path::Path;
 
 use cxx::UniquePtr;
+use log::trace;
 use mwalib::CorrelatorContext;
 use uvfits::UvfitsWriter;
 
-use crate::{CxxFlagMask, CxxImageSet};
+use crate::{pos::earth::LatLngHeight, CxxFlagMask, CxxImageSet};
 
 use self::error::UvfitsWriteError;
 
@@ -32,27 +33,34 @@ pub fn write_uvfits<'a>(
     baseline_flagmasks: &[UniquePtr<CxxFlagMask>],
     img_timestep_idxs: &[usize],
     img_coarse_chan_idxs: &[usize],
+    array_pos: Option<LatLngHeight>,
 ) -> Result<(), UvfitsWriteError> {
+    trace!("start write_uvfits");
+
     let mut uvfits_writer = UvfitsWriter::from_mwalib(
         filename,
-        &context,
-        &img_timestep_idxs,
-        &img_coarse_chan_idxs,
+        context,
+        img_timestep_idxs,
+        img_coarse_chan_idxs,
+        baseline_idxs,
+        array_pos,
     )?;
 
     let mut fits_file = uvfits_writer.open().unwrap();
 
     uvfits_writer.write_baseline_imgset_flagmasks(
         &mut fits_file,
-        &context,
-        &baseline_idxs,
-        &baseline_imgsets,
-        &baseline_flagmasks,
-        &img_timestep_idxs,
-        &img_coarse_chan_idxs,
+        context,
+        baseline_idxs,
+        baseline_imgsets,
+        baseline_flagmasks,
+        img_timestep_idxs,
+        img_coarse_chan_idxs,
     )?;
 
-    uvfits_writer.write_ants_from_mwalib(&context.metafits_context, None)?;
+    uvfits_writer.write_ants_from_mwalib(&context.metafits_context)?;
+
+    trace!("end write_uvfits");
 
     Ok(())
 }
@@ -107,7 +115,7 @@ mod tests {
         let mut baseline_flagmasks = init_baseline_flagmasks(
             &aoflagger,
             &context,
-            &img_coarse_chan_idxs,
+            img_coarse_chan_idxs,
             &img_timestep_idxs,
             Some(get_antenna_flags(&context)),
         );
@@ -116,7 +124,7 @@ mod tests {
         let baseline_imgsets = context_to_baseline_imgsets(
             &aoflagger,
             &context,
-            &img_coarse_chan_idxs,
+            img_coarse_chan_idxs,
             &img_timestep_idxs,
             Some(&mut baseline_flagmasks),
         );
@@ -127,7 +135,7 @@ mod tests {
         // run the strategy on the imagesets, and get the resulting flagmasks for each baseline
         flag_imgsets_existing(
             &aoflagger,
-            &strategy_filename,
+            strategy_filename,
             &baseline_imgsets,
             &mut baseline_flagmasks,
             true,
@@ -143,7 +151,8 @@ mod tests {
             &baseline_imgsets,
             &baseline_flagmasks,
             &img_timestep_idxs,
-            &img_coarse_chan_idxs,
+            img_coarse_chan_idxs,
+            None,
         )
         .unwrap();
 
