@@ -17,9 +17,7 @@
 
 use mwalib::{MetafitsContext, MWA_ALTITUDE_METRES, MWA_LATITUDE_RADIANS, MWA_LONGITUDE_RADIANS};
 // use rayon::prelude::*;
-use thiserror::Error;
-
-use super::ENH;
+use super::{error::ErfaError, ENH};
 
 /// The geodetic (x,y,z) coordinates of an antenna (a.k.a. tile or station). All
 /// units are in metres.
@@ -48,11 +46,11 @@ impl XyzGeodetic {
         }
     }
 
-    /// Convert [XyzGeodetic] coordinates at the MWA's latitude to [ENH]
-    /// coordinates.
-    pub fn to_enh_mwa(&self) -> ENH {
-        self.to_enh(MWA_LATITUDE_RADIANS)
-    }
+    // /// Convert [XyzGeodetic] coordinates at the MWA's latitude to [ENH]
+    // /// coordinates.
+    // pub fn to_enh_mwa(&self) -> ENH {
+    //     self.to_enh(MWA_LATITUDE_RADIANS)
+    // }
 
     /// For each [XyzGeodetic] pair, calculate an [XyzBaseline].
     pub fn get_baselines(xyz: &[Self]) -> Vec<XyzBaseline> {
@@ -75,6 +73,32 @@ impl XyzGeodetic {
     /// "input"; e.g. in the metafits file, Tile104 is often the first tile
     /// listed ("input" 0), Tile103 second ("input" 2), so the first baseline
     /// would naively be between Tile104 and Tile103.
+    pub fn get_tiles(context: &MetafitsContext, latitude_rad: f64) -> Vec<Self> {
+        context
+            .rf_inputs
+            .iter()
+            // There is an RF input for both tile polarisations. The ENH
+            // coordinates are the same for both polarisations of a tile; ignore
+            // the RF input if it's associated with Y.
+            .filter(|rf| matches!(rf.pol, mwalib::Pol::Y))
+            .map(|rf| {
+                ENH {
+                    e: rf.east_m,
+                    n: rf.north_m,
+                    h: rf.height_m,
+                }
+                .to_xyz(latitude_rad)
+            })
+            .collect()
+    }
+
+    /// For each tile listed in an mwalib context, calculate a [XyzGeodetic]
+    /// coordinate. Use default MWALib's MWA position to translate to xyz.
+    ///
+    /// Note that the RF inputs are ordered by antenna number, **not** the
+    /// "input"; e.g. in the metafits file, Tile104 is often the first tile
+    /// listed ("input" 0), Tile103 second ("input" 2), so the first baseline
+    /// would naively be between Tile104 and Tile103.
     pub fn get_tiles_mwalib(context: &MetafitsContext) -> Vec<Self> {
         context
             .rf_inputs
@@ -94,15 +118,15 @@ impl XyzGeodetic {
             .collect()
     }
 
-    /// For each tile listed in an mwalib context, calculate an [XyzBaseline].
-    ///
-    /// Note that the RF inputs are ordered by antenna number, **not** the
-    /// "input"; e.g. in the metafits file, Tile104 is often the first tile
-    /// listed ("input" 0), Tile103 second ("input" 2), so the first baseline
-    /// would naively be between Tile104 and Tile103.
-    pub fn get_baselines_mwalib(context: &MetafitsContext) -> Vec<XyzBaseline> {
-        Self::get_baselines(&Self::get_tiles_mwalib(context))
-    }
+    // /// For each tile listed in an mwalib context, calculate an [XyzBaseline].
+    // ///
+    // /// Note that the RF inputs are ordered by antenna number, **not** the
+    // /// "input"; e.g. in the metafits file, Tile104 is often the first tile
+    // /// listed ("input" 0), Tile103 second ("input" 2), so the first baseline
+    // /// would naively be between Tile104 and Tile103.
+    // pub fn get_baselines_mwalib(context: &MetafitsContext) -> Vec<XyzBaseline> {
+    //     Self::get_baselines(&Self::get_tiles_mwalib(context))
+    // }
 
     fn to_geocentric_inner(
         &self,
@@ -148,16 +172,25 @@ impl XyzGeodetic {
             MWA_ALTITUDE_METRES,
         )
     }
+
+    // /// Convert to an [`XyzBaseline`]
+    // ///
+    // pub fn to_baseline(&self) -> XyzBaseline {
+    //     XyzBaseline {
+    //         x: self.x,
+    //         y: self.y,
+    //         z: self.z,
+    //     }
+    // }
 }
 
-/// Convert [XyzGeodetic] coordinates to [UVW]s without having to form
-/// [XyzBaseline]s first.
-// pub fn xyzs_to_uvws(xyzs: &[XyzGeodetic], phase_centre: &HADec) -> Vec<UVW> {
+// /// Convert [XyzGeodetic] coordinates to [UVW]s without having to form
+// /// [XyzBaseline]s first.
+// pub fn tile_xyzs_to_uvws(xyzs: &[XyzGeodetic], phase_centre: &HADec) -> Vec<UVW> {
 //     let (s_ha, c_ha) = phase_centre.ha.sin_cos();
 //     let (s_dec, c_dec) = phase_centre.dec.sin_cos();
 //     // Get a UVW for each tile.
-//     let tile_uvws: Vec<UVW> = xyzs
-//         .iter()
+//     xyzs.iter()
 //         .map(|xyz| {
 //             let bl = XyzBaseline {
 //                 x: xyz.x,
@@ -166,13 +199,20 @@ impl XyzGeodetic {
 //             };
 //             UVW::from_xyz_inner(&bl, s_ha, c_ha, s_dec, c_dec)
 //         })
-//         .collect();
+//         .collect()
+// }
+
+// /// Convert [XyzGeodetic] coordinates to [UVW]s without having to form
+// /// [XyzBaseline]s first.
+// pub fn xyzs_to_uvws(xyzs: &[XyzGeodetic], phase_centre: &HADec) -> Vec<UVW> {
+//     // Get a UVW for each tile.
+//     let tile_uvws: Vec<UVW> = tile_xyzs_to_uvws(xyzs, phase_centre);
 //     // Take the difference of every pair of UVWs.
 //     let num_tiles = xyzs.len();
 //     let num_baselines = (num_tiles * (num_tiles - 1)) / 2;
 //     let mut bl_uvws = Vec::with_capacity(num_baselines);
 //     for i in 0..num_tiles {
-//         for j in i + 1..num_tiles {
+//         for j in i..num_tiles {
 //             let tile_1 = tile_uvws[i];
 //             let tile_2 = tile_uvws[j];
 //             let uvw_bl = UVW {
@@ -216,17 +256,17 @@ pub fn geodetic_to_geocentric_mwa(
     )
 }
 
-impl std::ops::Sub<XyzGeodetic> for XyzGeodetic {
-    type Output = XyzBaseline;
+// impl std::ops::Sub<XyzGeodetic> for XyzGeodetic {
+//     type Output = XyzBaseline;
 
-    fn sub(self, rhs: Self) -> XyzBaseline {
-        XyzBaseline {
-            x: self.x - rhs.x,
-            y: self.y - rhs.y,
-            z: self.z - rhs.z,
-        }
-    }
-}
+//     fn sub(self, rhs: Self) -> XyzBaseline {
+//         XyzBaseline {
+//             x: self.x - rhs.x,
+//             y: self.y - rhs.y,
+//             z: self.z - rhs.z,
+//         }
+//     }
+// }
 
 impl std::ops::Sub<&XyzGeodetic> for XyzGeodetic {
     type Output = XyzBaseline;
@@ -257,6 +297,17 @@ pub struct XyzBaseline {
     pub z: f64,
 }
 
+// impl XyzBaseline {
+//     /// Convert to an [`XyzGeodetic`]
+//     pub fn to_geodetic(&self) -> XyzGeodetic {
+//         XyzGeodetic {
+//             x: self.x,
+//             y: self.y,
+//             z: self.z,
+//         }
+//     }
+// }
+
 /// The geocentric (x,y,z) coordinates of an antenna (a.k.a. tile or station).
 /// All units are in metres.
 ///
@@ -342,7 +393,7 @@ impl XyzGeocentric {
             XyzGeocentric::get_geocentric_vector(longitude_rad, latitude_rad, height_metres)?;
         let (sin_longitude, cos_longitude) = (-longitude_rad).sin_cos();
         let geodetic = XyzGeocentric::to_geodetic_inner(
-            &self,
+            self,
             &geocentric_vector,
             sin_longitude,
             cos_longitude,
@@ -411,19 +462,6 @@ pub fn geocentric_to_geodetic_mwa(
 //     Ok(geodetics)
 // }
 
-#[derive(Error, Debug)]
-#[error(
-    "{source_file}:{source_line} Call to ERFA function {function} returned status code {status}"
-)]
-
-/// An error associated with ERFA.
-pub struct ErfaError {
-    source_file: &'static str,
-    source_line: u32,
-    status: i32,
-    function: &'static str,
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
